/*
 * Copyright 2011, Maarten Baert <maarten-baert@hotmail.com>
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef BUFFER_H
#define BUFFER_H

#include "SharedBuffers.h"

extern SharedBufferInterface shared_buffer_iterface;

class Buffer {
	
	private:
	char *data;
	unsigned int length, capacity;
	unsigned int pos;
	bool error;
	
	unsigned int id;
	
	public:
	// all functions are static because SharedBufferInterface uses function pointers
	
	static Buffer* Create();
	static void Destroy(Buffer* buffer);
	static unsigned int GetID(Buffer* buffer);
	static Buffer* Find(unsigned int id);
	
	static const char* ToString(Buffer* buffer);
	static char* GetData(Buffer* buffer);
	static unsigned int GetPos(Buffer* buffer);
	static unsigned int GetLength(Buffer* buffer);
	static bool IsAtEnd(Buffer* buffer);
	static bool GetError(Buffer* buffer);
	static void ClearError(Buffer* buffer);
	
	static void Clear(Buffer* buffer);
	static void SetPos(Buffer* buffer, unsigned int newpos);
	static void SetLength(Buffer* buffer, unsigned int newlength);
	
	static bool ReadFromFile(Buffer* buffer, const char* filename);
	static bool WriteToFile(Buffer* buffer, const char* filename);
	
	template<typename X>
	static X ReadX(Buffer* buffer);
	template<typename X>
	static void WriteX(Buffer* buffer, X value);
	
	static const char* ReadString(Buffer* buffer);
	static void WriteString(Buffer* buffer, const char* string);
	
	static void ReadData(Buffer* buffer, char* ptr, unsigned int bytes);
	static void WriteData(Buffer* buffer, const char* ptr, unsigned int bytes);
	
	static void ReadHex(Buffer* buffer, char* ptr, unsigned int bytes);
	static void WriteHex(Buffer* buffer, const char* ptr, unsigned int bytes);
	
	static void WriteBuffer(Buffer* buffer, Buffer* source);
	static void WriteBufferPart(Buffer* buffer, Buffer* source, unsigned int offset, unsigned int bytes);
	
};

#endif // BUFFER_H

